@@ -1,39 +1,31 @@
-# Copyright (C) 2015-2020  The Software Heritage developers
+# Copyright (C) 2015-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+import datetime
 import json
 import logging
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 from uuid import UUID
 
-from arrow import Arrow, utcnow
 import attr
-from psycopg2.extensions import AsIs
+from psycopg2.errors import CardinalityViolation
 import psycopg2.extras
 import psycopg2.pool
 
 from swh.core.db import BaseDb
 from swh.core.db.common import db_transaction
+from swh.scheduler.utils import utcnow
 
-from .exc import StaleData
-from .model import (
-    ListedOrigin,
-    ListedOriginPageToken,
-    Lister,
-    PaginatedListedOriginList,
-)
+from .exc import SchedulerException, StaleData, UnknownPolicy
+from .interface import ListedOriginPageToken, PaginatedListedOriginList
+from .model import ListedOrigin, Lister, OriginVisitStats, SchedulerMetrics
 
 logger = logging.getLogger(__name__)
 
 
-def adapt_arrow(arrow):
-    return AsIs("'%s'::timestamptz" % arrow.isoformat())
-
-
 psycopg2.extensions.register_adapter(dict, psycopg2.extras.Json)
-psycopg2.extensions.register_adapter(Arrow, adapt_arrow)
 psycopg2.extras.register_uuid()
 
 
@@ -139,6 +131,31 @@ class SchedulerBackend:
         cur.execute(query)
         return cur.fetchall()
 
+    @db_transaction()
+    def get_lister(
+        self, name: str, instance_name: Optional[str] = None, db=None, cur=None
+    ) -> Optional[Lister]:
+        """Retrieve information about the given instance of the lister from the
+        database.
+        """
+        if instance_name is None:
+            instance_name = ""
+
+        select_cols = ", ".join(Lister.select_columns())
+
+        query = f"""
+            select {select_cols} from listers
+              where (name, instance_name) = (%s, %s)
+        """
+
+        cur.execute(query, (name, instance_name))
+
+        ret = cur.fetchone()
+        if not ret:
+            return None
+
+        return Lister(**ret)
+
     @db_transaction()
     def get_or_create_lister(
         self, name: str, instance_name: Optional[str] = None, db=None, cur=None
@@ -286,12 +303,150 @@ class SchedulerBackend:
         origins = [ListedOrigin(**d) for d in cur]
 
         if len(origins) == limit:
-            page_token = (origins[-1].lister_id, origins[-1].url)
+            page_token = (str(origins[-1].lister_id), origins[-1].url)
         else:
             page_token = None
 
         return PaginatedListedOriginList(origins, page_token)
 
+    @db_transaction()
+    def grab_next_visits(
+        self,
+        visit_type: str,
+        count: int,
+        policy: str,
+        timestamp: Optional[datetime.datetime] = None,
+        db=None,
+        cur=None,
+    ) -> List[ListedOrigin]:
+        """Get at most the `count` next origins that need to be visited with
+        the `visit_type` loader according to the given scheduling `policy`.
+
+        This will mark the origins as scheduled in the origin_visit_stats
+        table, to avoid scheduling multiple visits to the same origin.
+
+        Arguments:
+          visit_type: type of visits to schedule
+          count: number of visits to schedule
+          policy: the scheduling policy used to select which visits to schedule
+          timestamp: the mocked timestamp at which we're recording that the visits are
+            being scheduled (defaults to the current time)
+        """
+        if timestamp is None:
+            timestamp = utcnow()
+
+        origin_select_cols = ", ".join(ListedOrigin.select_columns())
+
+        query_args: List[Any] = []
+
+        where_clauses = []
+
+        # "NOT enabled" = the lister said the origin no longer exists
+        where_clauses.append("enabled")
+
+        # Only schedule visits of the given type
+        where_clauses.append("visit_type = %s")
+        query_args.append(visit_type)
+
+        # Don't re-schedule visits if they're already scheduled but we haven't
+        # recorded a result yet, unless they've been scheduled more than a week
+        # ago (it probably means we've lost them in flight somewhere).
+        where_clauses.append(
+            """origin_visit_stats.last_scheduled IS NULL
+            OR origin_visit_stats.last_scheduled < GREATEST(
+              %s - '7 day'::interval,
+              origin_visit_stats.last_eventful,
+              origin_visit_stats.last_uneventful,
+              origin_visit_stats.last_failed,
+              origin_visit_stats.last_notfound
+            )
+        """
+        )
+        query_args.append(timestamp)
+
+        if policy == "oldest_scheduled_first":
+            order_by = "origin_visit_stats.last_scheduled NULLS FIRST"
+        elif policy == "never_visited_oldest_update_first":
+            # never visited origins have a NULL last_snapshot
+            where_clauses.append("origin_visit_stats.last_snapshot IS NULL")
+
+            # order by increasing last_update (oldest first)
+            where_clauses.append("listed_origins.last_update IS NOT NULL")
+            order_by = "listed_origins.last_update"
+        elif policy == "already_visited_order_by_lag":
+            # TODO: store "visit lag" in a materialized view?
+
+            # visited origins have a NOT NULL last_snapshot
+            where_clauses.append("origin_visit_stats.last_snapshot IS NOT NULL")
+
+            # ignore origins we have visited after the known last update
+            where_clauses.append("listed_origins.last_update IS NOT NULL")
+            where_clauses.append(
+                """
+              listed_origins.last_update
+              > GREATEST(
+                origin_visit_stats.last_eventful,
+                origin_visit_stats.last_uneventful
+              )
+            """
+            )
+
+            # order by decreasing visit lag
+            order_by = """\
+              listed_origins.last_update
+              - GREATEST(
+                origin_visit_stats.last_eventful,
+                origin_visit_stats.last_uneventful
+              )
+              DESC
+            """
+        else:
+            raise UnknownPolicy(f"Unknown scheduling policy {policy}")
+
+        select_query = f"""
+          SELECT
+            {origin_select_cols}
+          FROM
+            listed_origins
+          LEFT JOIN
+            origin_visit_stats USING (url, visit_type)
+          WHERE
+            ({") AND (".join(where_clauses)})
+          ORDER BY
+            {order_by}
+          LIMIT %s
+        """
+        query_args.append(count)
+
+        query = f"""
+            WITH selected_origins AS (
+               {select_query}
+            ),
+            update_stats AS (
+              INSERT INTO
+                origin_visit_stats (
+                  url, visit_type, last_scheduled
+                )
+              SELECT
+                url, visit_type, %s
+              FROM
+                selected_origins
+              ON CONFLICT (url, visit_type) DO UPDATE
+                SET last_scheduled = GREATEST(
+                  origin_visit_stats.last_scheduled,
+                  EXCLUDED.last_scheduled
+                )
+            )
+            SELECT
+              *
+            FROM
+              selected_origins
+        """
+        query_args.append(timestamp)
+
+        cur.execute(query, tuple(query_args))
+        return [ListedOrigin(**d) for d in cur]
+
     task_create_keys = [
         "type",
         "arguments",
@@ -339,7 +494,12 @@ class SchedulerBackend:
 
     @db_transaction()
     def set_status_tasks(
-        self, task_ids, status="disabled", next_run=None, db=None, cur=None
+        self,
+        task_ids: List[int],
+        status: str = "disabled",
+        next_run: Optional[datetime.datetime] = None,
+        db=None,
+        cur=None,
     ):
         """Set the tasks' status whose ids are listed.
 
@@ -348,7 +508,7 @@ class SchedulerBackend:
         if not task_ids:
             return
         query = ["UPDATE task SET status = %s"]
-        args = [status]
+        args: List[Any] = [status]
         if next_run:
             query.append(", next_run = %s")
             args.append(next_run)
@@ -437,34 +597,19 @@ class SchedulerBackend:
     @db_transaction()
     def peek_ready_tasks(
         self,
-        task_type,
-        timestamp=None,
-        num_tasks=None,
-        num_tasks_priority=None,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
         db=None,
         cur=None,
-    ):
-        """Fetch the list of ready tasks
-
-        Args:
-            task_type (str): filtering task per their type
-            timestamp (datetime.datetime): peek tasks that need to be executed
-                before that timestamp
-            num_tasks (int): only peek at num_tasks tasks (with no priority)
-            num_tasks_priority (int): only peek at num_tasks_priority
-                                      tasks (with priority)
-
-        Returns:
-            a list of tasks
-
-        """
+    ) -> List[Dict]:
         if timestamp is None:
             timestamp = utcnow()
 
         cur.execute(
-            """select * from swh_scheduler_peek_ready_tasks(
-                %s, %s, %s :: bigint, %s :: bigint)""",
-            (task_type, timestamp, num_tasks, num_tasks_priority),
+            """select * from swh_scheduler_peek_no_priority_tasks(
+                %s, %s, %s :: bigint)""",
+            (task_type, timestamp, num_tasks),
         )
         logger.debug("PEEK %s => %s" % (task_type, cur.rowcount))
         return cur.fetchall()
@@ -472,37 +617,61 @@ class SchedulerBackend:
     @db_transaction()
     def grab_ready_tasks(
         self,
-        task_type,
-        timestamp=None,
-        num_tasks=None,
-        num_tasks_priority=None,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
         db=None,
         cur=None,
-    ):
-        """Fetch the list of ready tasks, and mark them as scheduled
-
-        Args:
-            task_type (str): filtering task per their type
-            timestamp (datetime.datetime): grab tasks that need to be executed
-                before that timestamp
-            num_tasks (int): only grab num_tasks tasks (with no priority)
-            num_tasks_priority (int): only grab oneshot num_tasks tasks (with
-                                      priorities)
-
-        Returns:
-            a list of tasks
-
-        """
+    ) -> List[Dict]:
         if timestamp is None:
             timestamp = utcnow()
         cur.execute(
             """select * from swh_scheduler_grab_ready_tasks(
-                 %s, %s, %s :: bigint, %s :: bigint)""",
-            (task_type, timestamp, num_tasks, num_tasks_priority),
+                 %s, %s, %s :: bigint)""",
+            (task_type, timestamp, num_tasks),
         )
         logger.debug("GRAB %s => %s" % (task_type, cur.rowcount))
         return cur.fetchall()
 
+    @db_transaction()
+    def peek_ready_priority_tasks(
+        self,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
+        db=None,
+        cur=None,
+    ) -> List[Dict]:
+        if timestamp is None:
+            timestamp = utcnow()
+
+        cur.execute(
+            """select * from swh_scheduler_peek_any_ready_priority_tasks(
+                %s, %s, %s :: bigint)""",
+            (task_type, timestamp, num_tasks),
+        )
+        logger.debug("PEEK %s => %s", task_type, cur.rowcount)
+        return cur.fetchall()
+
+    @db_transaction()
+    def grab_ready_priority_tasks(
+        self,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
+        db=None,
+        cur=None,
+    ) -> List[Dict]:
+        if timestamp is None:
+            timestamp = utcnow()
+        cur.execute(
+            """select * from swh_scheduler_grab_any_ready_priority_tasks(
+                 %s, %s, %s :: bigint)""",
+            (task_type, timestamp, num_tasks),
+        )
+        logger.debug("GRAB %s => %s", task_type, cur.rowcount)
+        return cur.fetchall()
+
     task_run_create_keys = ["task", "backend_id", "scheduled", "metadata"]
 
     @db_transaction()
@@ -728,6 +897,113 @@ class SchedulerBackend:
         return cur.fetchall()
 
     @db_transaction()
-    def get_priority_ratios(self, db=None, cur=None):
-        cur.execute("select id, ratio from priority_ratio")
-        return {row["id"]: row["ratio"] for row in cur.fetchall()}
+    def origin_visit_stats_upsert(
+        self, origin_visit_stats: Iterable[OriginVisitStats], db=None, cur=None
+    ) -> None:
+        pk_cols = OriginVisitStats.primary_key_columns()
+        insert_cols, insert_meta = OriginVisitStats.insert_columns_and_metavars()
+
+        query = f"""
+            INSERT into origin_visit_stats AS ovi ({", ".join(insert_cols)})
+            VALUES %s
+            ON CONFLICT ({", ".join(pk_cols)}) DO UPDATE
+            SET last_eventful = coalesce(excluded.last_eventful, ovi.last_eventful),
+                last_uneventful = coalesce(excluded.last_uneventful, ovi.last_uneventful),
+                last_failed = coalesce(excluded.last_failed, ovi.last_failed),
+                last_notfound = coalesce(excluded.last_notfound, ovi.last_notfound),
+                last_snapshot = coalesce(excluded.last_snapshot, ovi.last_snapshot)
+        """  # noqa
+
+        try:
+            psycopg2.extras.execute_values(
+                cur=cur,
+                sql=query,
+                argslist=(
+                    attr.asdict(visit_stats) for visit_stats in origin_visit_stats
+                ),
+                template=f"({', '.join(insert_meta)})",
+                page_size=1000,
+                fetch=False,
+            )
+        except CardinalityViolation as e:
+            raise SchedulerException(repr(e))
+
+    @db_transaction()
+    def origin_visit_stats_get(
+        self, ids: Iterable[Tuple[str, str]], db=None, cur=None
+    ) -> List[OriginVisitStats]:
+        if not ids:
+            return []
+        primary_keys = tuple((origin, visit_type) for (origin, visit_type) in ids)
+        query = format_query(
+            """
+            SELECT {keys}
+            FROM (VALUES %s) as stats(url, visit_type)
+            INNER JOIN origin_visit_stats USING (url, visit_type)
+        """,
+            OriginVisitStats.select_columns(),
+        )
+        rows = psycopg2.extras.execute_values(
+            cur=cur, sql=query, argslist=primary_keys, fetch=True
+        )
+        return [OriginVisitStats(**row) for row in rows]
+
+    @db_transaction()
+    def update_metrics(
+        self,
+        lister_id: Optional[UUID] = None,
+        timestamp: Optional[datetime.datetime] = None,
+        db=None,
+        cur=None,
+    ) -> List[SchedulerMetrics]:
+        """Update the performance metrics of this scheduler instance.
+
+        Returns the updated metrics.
+
+        Args:
+          lister_id: if passed, update the metrics only for this lister instance
+          timestamp: if passed, the date at which we're updating the metrics,
+            defaults to the database NOW()
+        """
+        query = format_query(
+            "SELECT {keys} FROM update_metrics(%s, %s)",
+            SchedulerMetrics.select_columns(),
+        )
+        cur.execute(query, (lister_id, timestamp))
+        return [SchedulerMetrics(**row) for row in cur.fetchall()]
+
+    @db_transaction()
+    def get_metrics(
+        self,
+        lister_id: Optional[UUID] = None,
+        visit_type: Optional[str] = None,
+        db=None,
+        cur=None,
+    ) -> List[SchedulerMetrics]:
+        """Retrieve the performance metrics of this scheduler instance.
+
+        Args:
+          lister_id: filter the metrics for this lister instance only
+          visit_type: filter the metrics for this visit type only
+        """
+
+        where_filters = []
+        where_args = []
+        if lister_id:
+            where_filters.append("lister_id = %s")
+            where_args.append(str(lister_id))
+        if visit_type:
+            where_filters.append("visit_type = %s")
+            where_args.append(visit_type)
+
+        where_clause = ""
+        if where_filters:
+            where_clause = f"where {' and '.join(where_filters)}"
+
+        query = format_query(
+            "SELECT {keys} FROM scheduler_metrics %s" % where_clause,
+            SchedulerMetrics.select_columns(),
+        )
+
+        cur.execute(query, tuple(where_args))
+        return [SchedulerMetrics(**row) for row in cur.fetchall()]
