@@ -1,20 +1,36 @@
-# Copyright (C) 2015-2020  The Software Heritage developers
+# Copyright (C) 2015-2021  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-from typing import Any, Dict, Iterable, List, Optional
+
+import datetime
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 from uuid import UUID
 
 from typing_extensions import Protocol, runtime_checkable
 
 from swh.core.api import remote_api_endpoint
-from swh.scheduler.model import (
-    ListedOrigin,
-    ListedOriginPageToken,
-    Lister,
-    PaginatedListedOriginList,
-)
+from swh.core.api.classes import PagedResult
+from swh.scheduler.model import ListedOrigin, Lister, OriginVisitStats, SchedulerMetrics
+
+ListedOriginPageToken = Tuple[str, str]
+
+
+class PaginatedListedOriginList(PagedResult[ListedOrigin, ListedOriginPageToken]):
+    """A list of listed origins, with a continuation token"""
+
+    def __init__(
+        self,
+        results: List[ListedOrigin],
+        next_page_token: Union[None, ListedOriginPageToken, List[str]],
+    ):
+        parsed_next_page_token: Optional[Tuple[str, str]] = None
+        if next_page_token is not None:
+            if len(next_page_token) != 2:
+                raise TypeError("Expected Tuple[str, str] or list of size 2.")
+            parsed_next_page_token = tuple(next_page_token)  # type: ignore
+        super().__init__(results, parsed_next_page_token)
 
 
 @runtime_checkable
@@ -78,7 +94,12 @@ class SchedulerInterface(Protocol):
         ...
 
     @remote_api_endpoint("task/set_status")
-    def set_status_tasks(self, task_ids, status="disabled", next_run=None):
+    def set_status_tasks(
+        self,
+        task_ids: List[int],
+        status: str = "disabled",
+        next_run: Optional[datetime.datetime] = None,
+    ):
         """Set the tasks' status whose ids are listed.
 
         If given, also set the next_run date.
@@ -112,17 +133,59 @@ class SchedulerInterface(Protocol):
 
     @remote_api_endpoint("task/peek_ready")
     def peek_ready_tasks(
-        self, task_type, timestamp=None, num_tasks=None, num_tasks_priority=None,
-    ):
-        """Fetch the list of ready tasks
+        self,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
+    ) -> List[Dict]:
+        """Fetch the list of tasks (with no priority) to be scheduled.
 
         Args:
-            task_type (str): filtering task per their type
-            timestamp (datetime.datetime): peek tasks that need to be executed
+            task_type: filtering task per their type
+            timestamp: peek tasks that need to be executed
                 before that timestamp
-            num_tasks (int): only peek at num_tasks tasks (with no priority)
-            num_tasks_priority (int): only peek at num_tasks_priority
-                                      tasks (with priority)
+            num_tasks: only peek at num_tasks tasks (with no priority)
+
+        Returns:
+            the list of tasks which would be scheduled
+
+        """
+        ...
+
+    @remote_api_endpoint("task/grab_ready")
+    def grab_ready_tasks(
+        self,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
+    ) -> List[Dict]:
+        """Fetch and schedule the list of tasks (with no priority) ready to be scheduled.
+
+        Args:
+            task_type: filtering task per their type
+            timestamp: grab tasks that need to be executed
+                before that timestamp
+            num_tasks: only grab num_tasks tasks (with no priority)
+
+        Returns:
+            the list of scheduled tasks
+
+        """
+        ...
+
+    @remote_api_endpoint("task/peek_ready_with_priority")
+    def peek_ready_priority_tasks(
+        self,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
+    ) -> List[Dict]:
+        """Fetch list of tasks (with any priority) ready to be scheduled.
+
+        Args:
+            task_type: filtering task per their type
+            timestamp: peek tasks that need to be executed before that timestamp
+            num_tasks: only peek at num_tasks tasks (with no priority)
 
         Returns:
             a list of tasks
@@ -130,19 +193,20 @@ class SchedulerInterface(Protocol):
         """
         ...
 
-    @remote_api_endpoint("task/grab_ready")
-    def grab_ready_tasks(
-        self, task_type, timestamp=None, num_tasks=None, num_tasks_priority=None,
-    ):
-        """Fetch the list of ready tasks, and mark them as scheduled
+    @remote_api_endpoint("task/grab_ready_with_priority")
+    def grab_ready_priority_tasks(
+        self,
+        task_type: str,
+        timestamp: Optional[datetime.datetime] = None,
+        num_tasks: Optional[int] = None,
+    ) -> List[Dict]:
+        """Fetch and schedule the list of tasks (with any priority) ready to be scheduled.
 
         Args:
-            task_type (str): filtering task per their type
-            timestamp (datetime.datetime): grab tasks that need to be executed
+            task_type: filtering task per their type
+            timestamp: grab tasks that need to be executed
                 before that timestamp
-            num_tasks (int): only grab num_tasks tasks (with no priority)
-            num_tasks_priority (int): only grab oneshot num_tasks tasks (with
-                                      priorities)
+            num_tasks: only grab num_tasks tasks (with no priority)
 
         Returns:
             a list of tasks
@@ -259,6 +323,15 @@ class SchedulerInterface(Protocol):
         """Search task run for a task id"""
         ...
 
+    @remote_api_endpoint("lister/get")
+    def get_lister(
+        self, name: str, instance_name: Optional[str] = None
+    ) -> Optional[Lister]:
+        """Retrieve information about the given instance of the lister from the
+        database.
+        """
+        ...
+
     @remote_api_endpoint("lister/get_or_create")
     def get_or_create_lister(
         self, name: str, instance_name: Optional[str] = None
@@ -309,6 +382,74 @@ class SchedulerInterface(Protocol):
         """
         ...
 
-    @remote_api_endpoint("priority_ratios/get")
-    def get_priority_ratios(self):
+    @remote_api_endpoint("origins/grab_next")
+    def grab_next_visits(
+        self,
+        visit_type: str,
+        count: int,
+        policy: str,
+        timestamp: Optional[datetime.datetime] = None,
+    ) -> List[ListedOrigin]:
+        """Get at most the `count` next origins that need to be visited with
+        the `visit_type` loader according to the given scheduling `policy`.
+
+        This will mark the origins as scheduled in the origin_visit_stats
+        table, to avoid scheduling multiple visits to the same origin.
+
+        Arguments:
+          visit_type: type of visits to schedule
+          count: number of visits to schedule
+          policy: the scheduling policy used to select which visits to schedule
+          timestamp: the mocked timestamp at which we're recording that the visits are
+            being scheduled (defaults to the current time)
+        """
+        ...
+
+    @remote_api_endpoint("visit_stats/upsert")
+    def origin_visit_stats_upsert(
+        self, origin_visit_stats: Iterable[OriginVisitStats]
+    ) -> None:
+        """Create a new origin visit stats
+        """
+        ...
+
+    @remote_api_endpoint("visit_stats/get")
+    def origin_visit_stats_get(
+        self, ids: Iterable[Tuple[str, str]]
+    ) -> List[OriginVisitStats]:
+        """Retrieve the stats for an origin with a given visit type
+
+        If some visit_stats are not found, they are filtered out of the result. So the
+        output list may be of length inferior to the length of the input list.
+
+        """
+        ...
+
+    @remote_api_endpoint("scheduler_metrics/update")
+    def update_metrics(
+        self,
+        lister_id: Optional[UUID] = None,
+        timestamp: Optional[datetime.datetime] = None,
+    ) -> List[SchedulerMetrics]:
+        """Update the performance metrics of this scheduler instance.
+
+        Returns the updated metrics.
+
+        Args:
+          lister_id: if passed, update the metrics only for this lister instance
+          timestamp: if passed, the date at which we're updating the metrics,
+            defaults to the database NOW()
+        """
+        ...
+
+    @remote_api_endpoint("scheduler_metrics/get")
+    def get_metrics(
+        self, lister_id: Optional[UUID] = None, visit_type: Optional[str] = None
+    ) -> List[SchedulerMetrics]:
+        """Retrieve the performance metrics of this scheduler instance.
+
+        Args:
+          lister_id: filter the metrics for this lister instance only
+          visit_type: filter the metrics for this visit type only
+        """
         ...
