@@ -1,15 +1,56 @@
+# Copyright (C) 2019-2021 The Software Heritage developers
+# See the AUTHORS file at the top-level directory of this distribution
+# License: GNU General Public License version 3, or any later version
+# See top-level LICENSE file for more information
+
+"""Module in charge of testing the scheduler runner module"""
+
 from itertools import count
 from time import sleep
 
 from celery.result import AsyncResult, GroupResult
+from kombu import Exchange, Queue
 import pytest
 
 from swh.scheduler.celery_backend.runner import run_ready_tasks
+from swh.scheduler.tests.tasks import (
+    TASK_ADD,
+    TASK_ECHO,
+    TASK_ERROR,
+    TASK_MULTIPING,
+    TASK_PING,
+)
 from swh.scheduler.utils import create_task_dict
 
+# Queues to subscribe. Due to the rerouting of high priority tasks, this module requires
+# to declare all queues/task names
+TEST_QUEUES = [
+    "celery",
+    TASK_ECHO,
+    TASK_ERROR,
+    TASK_PING,
+    TASK_ADD,
+    TASK_MULTIPING,
+    # and the high priority queue
+    f"save_code_now:{TASK_ADD}",
+]
+
+
+@pytest.fixture(scope="session")
+def swh_scheduler_celery_app(swh_scheduler_celery_app):
+    swh_scheduler_celery_app.add_defaults(
+        {
+            "task_queues": [
+                Queue(queue, Exchange(queue), routing_key=queue)
+                for queue in TEST_QUEUES
+            ],
+        }
+    )
+    return swh_scheduler_celery_app
+
 
 def test_ping(swh_scheduler_celery_app, swh_scheduler_celery_worker):
-    res = swh_scheduler_celery_app.send_task("swh.scheduler.tests.tasks.ping")
+    res = swh_scheduler_celery_app.send_task(TASK_PING)
     assert res
     res.wait()
     assert res.successful()
@@ -17,9 +58,7 @@ def test_ping(swh_scheduler_celery_app, swh_scheduler_celery_worker):
 
 
 def test_ping_with_kw(swh_scheduler_celery_app, swh_scheduler_celery_worker):
-    res = swh_scheduler_celery_app.send_task(
-        "swh.scheduler.tests.tasks.ping", kwargs={"a": 1}
-    )
+    res = swh_scheduler_celery_app.send_task(TASK_PING, kwargs={"a": 1})
     assert res
     res.wait()
     assert res.successful()
@@ -28,9 +67,7 @@ def test_ping_with_kw(swh_scheduler_celery_app, swh_scheduler_celery_worker):
 
 def test_multiping(swh_scheduler_celery_app, swh_scheduler_celery_worker):
     "Test that a task that spawns subtasks (group) works"
-    res = swh_scheduler_celery_app.send_task(
-        "swh.scheduler.tests.tasks.multiping", kwargs={"n": 5}
-    )
+    res = swh_scheduler_celery_app.send_task(TASK_MULTIPING, kwargs={"n": 5})
     assert res
 
     res.wait()
@@ -59,7 +96,7 @@ def test_scheduler_fixture(
     task_type = swh_scheduler.get_task_type("swh-test-ping")
 
     assert task_type
-    assert task_type["backend_name"] == "swh.scheduler.tests.tasks.ping"
+    assert task_type["backend_name"] == TASK_PING
 
     swh_scheduler.create_tasks([create_task_dict("swh-test-ping", "oneshot")])
 
@@ -70,20 +107,92 @@ def test_scheduler_fixture(
         AsyncResult(id=task["backend_id"]).get()
 
 
-def test_task_return_value(
+def test_run_ready_task_standard(
     swh_scheduler_celery_app, swh_scheduler_celery_worker, swh_scheduler
 ):
-    task_type = swh_scheduler.get_task_type("swh-test-add")
+    """Ensure scheduler runner schedules tasks ready for scheduling"""
+    task_type_name, backend_name = "swh-test-add", TASK_ADD
+    task_type = swh_scheduler.get_task_type(task_type_name)
     assert task_type
-    assert task_type["backend_name"] == "swh.scheduler.tests.tasks.add"
+    assert task_type["backend_name"] == backend_name
 
-    swh_scheduler.create_tasks([create_task_dict("swh-test-add", "oneshot", 12, 30)])
+    task_inputs = [
+        ("oneshot", (12, 30)),
+        ("oneshot", (20, 10)),
+        ("recurring", (30, 10)),
+    ]
+
+    tasks = swh_scheduler.create_tasks(
+        create_task_dict(task_type_name, policy, *args)
+        for (policy, args) in task_inputs
+    )
+
+    assert len(tasks) == len(task_inputs)
+
+    task_ids = set()
+    for task in tasks:
+        assert task["status"] == "next_run_not_scheduled"
+        assert task["priority"] is None
+        task_ids.add(task["id"])
 
     backend_tasks = run_ready_tasks(swh_scheduler, swh_scheduler_celery_app)
-    assert len(backend_tasks) == 1
-    task = backend_tasks[0]
-    value = AsyncResult(id=task["backend_id"]).get()
-    assert value == 42
+    assert len(backend_tasks) == len(tasks)
+
+    scheduled_tasks = swh_scheduler.search_tasks(task_type=task_type_name)
+    assert len(scheduled_tasks) == len(tasks)
+    for task in scheduled_tasks:
+        assert task["status"] == "next_run_scheduled"
+        assert task["id"] in task_ids
+
+    # Ensure each task is indeed scheduled to the queue backend
+    for i, (_, args) in enumerate(task_inputs):
+        task = backend_tasks[i]
+        value = AsyncResult(id=task["backend_id"]).get()
+        assert value == sum(args)
+
+
+def test_run_ready_task_with_priority(
+    swh_scheduler_celery_app, swh_scheduler_celery_worker, swh_scheduler
+):
+    """Ensure scheduler runner schedules priority tasks ready for scheduling"""
+    task_type_name, backend_name = "swh-test-add", TASK_ADD
+    task_type = swh_scheduler.get_task_type(task_type_name)
+    assert task_type
+    assert task_type["backend_name"] == backend_name
+
+    task_inputs = [
+        ("oneshot", (10, 22), "low"),
+        ("oneshot", (20, 10), "normal"),
+        ("recurring", (30, 10), "high"),
+    ]
+
+    tasks = swh_scheduler.create_tasks(
+        create_task_dict(task_type_name, policy, *args, priority=priority)
+        for (policy, args, priority) in task_inputs
+    )
+
+    assert len(tasks) == len(task_inputs)
+
+    task_ids = set()
+    for task in tasks:
+        assert task["status"] == "next_run_not_scheduled"
+        assert task["priority"] is not None
+        task_ids.add(task["id"])
+
+    backend_tasks = run_ready_tasks(swh_scheduler, swh_scheduler_celery_app)
+    assert len(backend_tasks) == len(tasks)
+
+    scheduled_tasks = swh_scheduler.search_tasks(task_type=task_type_name)
+    assert len(scheduled_tasks) == len(tasks)
+    for task in scheduled_tasks:
+        assert task["status"] == "next_run_scheduled"
+        assert task["id"] in task_ids
+
+    # Ensure each priority task is indeed scheduled to the queue backend
+    for i, (_, args, _) in enumerate(task_inputs):
+        task = backend_tasks[i]
+        value = AsyncResult(id=task["backend_id"]).get()
+        assert value == sum(args)
 
 
 def test_task_exception(
@@ -91,7 +200,7 @@ def test_task_exception(
 ):
     task_type = swh_scheduler.get_task_type("swh-test-error")
     assert task_type
-    assert task_type["backend_name"] == "swh.scheduler.tests.tasks.error"
+    assert task_type["backend_name"] == TASK_ERROR
 
     swh_scheduler.create_tasks([create_task_dict("swh-test-error", "oneshot")])
 
@@ -108,7 +217,7 @@ def test_statsd(swh_scheduler_celery_app, swh_scheduler_celery_worker, mocker):
     m = mocker.patch("swh.scheduler.task.Statsd._send_to_server")
     mocker.patch("swh.scheduler.task.ts", side_effect=count())
     mocker.patch("swh.core.statsd.monotonic", side_effect=count())
-    res = swh_scheduler_celery_app.send_task("swh.scheduler.tests.tasks.echo")
+    res = swh_scheduler_celery_app.send_task(TASK_ECHO)
     assert res
     res.wait()
     assert res.successful()
@@ -143,9 +252,7 @@ def test_statsd_with_status(
     m = mocker.patch("swh.scheduler.task.Statsd._send_to_server")
     mocker.patch("swh.scheduler.task.ts", side_effect=count())
     mocker.patch("swh.core.statsd.monotonic", side_effect=count())
-    res = swh_scheduler_celery_app.send_task(
-        "swh.scheduler.tests.tasks.echo", kwargs={"status": "eventful"}
-    )
+    res = swh_scheduler_celery_app.send_task(TASK_ECHO, kwargs={"status": "eventful"})
     assert res
     res.wait()
     assert res.successful()
