@@ -1,10 +1,8 @@
-# Copyright (C) 2019  The Software Heritage developers
+# Copyright (C) 2019-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-import copy
-
 import pytest
 import yaml
 
@@ -33,79 +31,60 @@ def prepare_config_file(tmpdir, content, name="config.yml"):
     return str(config_path)
 
 
-def test_load_and_check_config_no_configuration():
+@pytest.mark.parametrize("scheduler_class", [None, ""])
+def test_load_and_check_config_no_configuration(scheduler_class):
     """Inexistent configuration files raises"""
-    with pytest.raises(EnvironmentError) as e:
-        load_and_check_config(None)
+    with pytest.raises(EnvironmentError, match="Configuration file must be defined"):
+        load_and_check_config(scheduler_class)
 
-    assert e.value.args[0] == "Configuration file must be defined"
 
+def test_load_and_check_config_inexistent_fil():
+    """Inexistent config filepath should raise"""
     config_path = "/some/inexistent/config.yml"
-    with pytest.raises(FileNotFoundError) as e:
+    expected_error = f"Configuration file {config_path} does not exist"
+    with pytest.raises(FileNotFoundError, match=expected_error):
         load_and_check_config(config_path)
 
-    assert e.value.args[0] == "Configuration file %s does not exist" % (config_path,)
-
 
 def test_load_and_check_config_wrong_configuration(tmpdir):
     """Wrong configuration raises"""
     config_path = prepare_config_file(tmpdir, "something: useless")
-    with pytest.raises(KeyError) as e:
+    with pytest.raises(KeyError, match="Missing '%scheduler' configuration"):
         load_and_check_config(config_path)
 
-    assert e.value.args[0] == "Missing '%scheduler' configuration"
-
 
 def test_load_and_check_config_remote_config_local_type_raise(tmpdir):
-    """'local' configuration without 'local' storage raises"""
-    config = {"scheduler": {"cls": "remote", "args": {}}}
+    """Configuration without 'local' storage is rejected"""
+    config = {"scheduler": {"cls": "remote"}}
     config_path = prepare_config_file(tmpdir, config)
-    with pytest.raises(ValueError) as e:
-        load_and_check_config(config_path, type="local")
-
-    assert (
-        e.value.args[0] == "The scheduler backend can only be started with a 'local'"
-        " configuration"
+    expected_error = (
+        "The scheduler backend can only be started with a 'local'" " configuration"
     )
+    with pytest.raises(ValueError, match=expected_error):
+        load_and_check_config(config_path, type="local")
 
 
 def test_load_and_check_config_local_incomplete_configuration(tmpdir):
     """Incomplete 'local' configuration should raise"""
-    config = {
-        "scheduler": {
-            "cls": "local",
-            "args": {"db": "database", "something": "needed-for-test",},
-        }
-    }
+    config = {"scheduler": {"cls": "local", "something": "needed-for-test",}}
 
-    for key in ["db", "args"]:
-        c = copy.deepcopy(config)
-        if key == "db":
-            source = c["scheduler"]["args"]
-        else:
-            source = c["scheduler"]
-        source.pop(key)
-        config_path = prepare_config_file(tmpdir, c)
-        with pytest.raises(KeyError) as e:
-            load_and_check_config(config_path)
-
-        assert (
-            e.value.args[0] == "Invalid configuration; missing '%s' config entry" % key
-        )
+    config_path = prepare_config_file(tmpdir, config)
+    expected_error = "Invalid configuration; missing 'db' config entry"
+    with pytest.raises(KeyError, match=expected_error):
+        load_and_check_config(config_path)
 
 
 def test_load_and_check_config_local_config_fine(tmpdir):
     """Local configuration is fine"""
-    config = {"scheduler": {"cls": "local", "args": {"db": "db",}}}
+    config = {"scheduler": {"cls": "local", "db": "db",}}
     config_path = prepare_config_file(tmpdir, config)
     cfg = load_and_check_config(config_path, type="local")
     assert cfg == config
 
 
 def test_load_and_check_config_remote_config_fine(tmpdir):
-    """'Remote configuration is fine"""
-    config = {"scheduler": {"cls": "remote", "args": {}}}
+    """Remote configuration is fine"""
+    config = {"scheduler": {"cls": "remote"}}
     config_path = prepare_config_file(tmpdir, config)
     cfg = load_and_check_config(config_path, type="any")
-
     assert cfg == config
